/*
 * AN00220_app_phase_aligned_example.h
 *
 *  Created on: 22. apr. 2021
 *      Author: teig
 */

#ifndef AN00220_APP_PHASE_ALIGNED_EXAMPLE_H_
    #define AN00220_APP_PHASE_ALIGNED_EXAMPLE_H_

    #ifndef MIC_ARRAY_WORD_LENGTH_SHORT
        #error MIC_ARRAY_WORD_LENGTH_SHORT not defined
    #else // in mic_array_frame.h lib_mic_array
        #if (MIC_ARRAY_WORD_LENGTH_SHORT==0) // THIS
            typedef int32_t mic_array_word_t;
            #define MIC_ARRAY_WORD_DB(arg) int32_dB(arg)
            #define MIC_ARRAY_WORD_MIN INT_MIN
            #define MIC_ARRAY_WORD_MAX INT_MAX
        #elif (MIC_ARRAY_WORD_LENGTH_SHORT==1)
            typedef int16_t mic_array_word_t;
            #define MIC_ARRAY_WORD_DB(arg) int16_dB(arg)
            #define MIC_ARRAY_WORD_MIN SHRT_MIN
            #define MIC_ARRAY_WORD_MAX SHRT_MAX
        #endif
    #endif

    typedef struct mic_result_t {
        unsigned         cnt;
        mic_array_word_t max;
        mic_array_word_t min;
    } mic_result_t;

    typedef interface mic_result_if {
        void send_mic_data (const mic_result_t mic_result);
    } mic_result_if;

    #ifndef MIC_ARRAY_NUM_MICS
        #error mic_array_conf.h not seen
    #endif

    #define NUM_4MIC_DECIMATOR_CHANS 4 // Simply because mic_array_decimate_to_pcm_4ch takes 4 mics
    #define NUM_4MIC_DECIMATORS      (MIC_ARRAY_NUM_MICS / NUM_4MIC_DECIMATOR_CHANS) // 8 channels requires 2 decimators:
    //                               // 2/1 when MIC_ARRAY_NUM_MICS == 8/4
    #define FRAME_BUFFER_COUNT       2 // AN00220/AN00219: The minimum of 2 will suffice for these examples (when DECIMATOR_NO_FRAME_OVERLAP)

    #define NUM_DATA_X (NUM_4MIC_DECIMATORS         * NUM_4MIC_DECIMATOR_CHANS) // 8/4 when MIC_ARRAY_NUM_MICS == 8/4
    #define NUM_DATA_Y (THIRD_STAGE_COEFS_PER_STAGE * DECIMATION_FACTOR)  // 192 = 32*6

    void example_task (
            streaming chanend ds_output_is_ptr_chan [NUM_4MIC_DECIMATORS], // [2/1] when MIC_ARRAY_NUM_MICS == 8/4
            int               data [NUM_DATA_X] [NUM_DATA_Y], // [8/4] [192] when MIC_ARRAY_NUM_MICS == 8/4
            out port          p_scope, // (*)
            //
            #if (MY_CONFIG == CONFIG_USE_INTERFACE)
                client mic_result_if if_mic_result);
            #elif (MY_CONFIG == CONFIG_USE_CHAN)
                chanend c_mic_result);
            #endif

            // (*) Having this as a param instead of being a global, as in the AN00220 example, yields the same
            // amount of all resources, on the byte, but the XTA timing analysis is _a_little_ worse. Strange. See files
            // "2021 04 23 09.40 build data global to example.txt"
            //     xta: /Users/teig/workspace/lib_mic_array/src/decimate_to_pcm_4ch.S:30: warning: route(4)     Pass with 4 unknowns, Num Paths: 1, Slack: 994.0 ns, Required: 2.6 us, Worst: 1.6 us, Min Core Frequency: 309 MHz
            // "2021 04 23 09.40 build data param to example.txt"                                                                                          ########                                                       #######
            //     xta: /Users/teig/workspace/lib_mic_array/src/decimate_to_pcm_4ch.S:30: warning: route(4)     Pass with 4 unknowns, Num Paths: 1, Slack: 984.0 ns, Required: 2.6 us, Worst: 1.6 us, Min Core Frequency: 311 MHz
#else
    #error NESTED .h FILE
#endif /* AN00220_APP_PHASE_ALIGNED_EXAMPLE_H_ */
