/*
 * _print_macros_xc.h
 *
 *  Created on: 12. mars 2021
 *      Author: teig
 */


#ifndef PRINT_MACROS_XC_H_
    #define PRINT_MACROS_XC_H_

    #ifndef DO_WARNINGS
        #define DO_WARNINGS 0
    #endif

    #if (DEBUG_PRINT_GLOBAL_APP==1)
        #if ((WARNINGS==1) and (DO_WARNINGS==1))
            #warning Unit must be connected to xTIMEcomposer. Not for off-line battery usage!
        #endif
    #endif

    #if not defined DEBUG_PRINT_GLOBAL_APP
        #error include _globals before this
    #elif (DEBUG_PRINT_GLOBAL_APP == 0)
        #if ((WARNINGS==1) and (DO_WARNINGS==1))
            #warning No printing
        #endif
        #define debug_print(fmt, ...)

    #elif (((defined DEBUG_PRINT_ENABLE_ALL) and (DEBUG_PRINT_ENABLE_ALL==1)) or \
         ((defined DEBUG_PRINT_ENABLE)     and (DEBUG_PRINT_ENABLE==1    )))

        #if ((WARNINGS==1) and (DO_WARNINGS==1))
            #warning Using /lib_logging (that is why sprintf is used when floats)
        #endif

        #include "debug_conf.h"
        #include "debug_print.h"

        // In makefile:
        //   USED_MODULES = .. lib_logging ..
        //   XCC_FLAGS += -DDEBUG_PRINT_ENABLE=1

        /*
        AN00239: https://www.xmos.ai/download/AN00239:-Using-the-logging-library(1.0.0rc1).pdf

        A limited functionality version of printf that is low memory.
        This function works like C-standard printf except that it only accepts d, x, s, u and c format
        specifiers with no conversions.
        The p format specifier is treated the same as a x.
        The capital version of each format specifier performs the same as the lower case equivalent.
        Any alignment or padding characters are simply ignored.
        The function uses the functions from print.h to do the underlying printing.
        Unlike printf this function has no return temp_degC.
        Whether the function does any output can be controlled via defines such as DEBUG_PRINT_ENABLE or
        DEBUG_PRINT_ENABLE_[debug unit name] in the applications debug_conf.h
        */

       #define debug_print(fmt, ...) do { if(DO_DEBUG_PRINT) debug_printf(fmt, __VA_ARGS__); } while (0) // In /lib_logging/api/debug_print.h

    #elif (DO_DEBUG_PRINT==1)
        #if ((WARNINGS==1) and (DO_WARNINGS==1))
            #warning jtag printing not as reliable
        #endif
        // See https://stackoverflow.com/questions/1644868/define-macro-for-debug-printing-in-c
        #define debug_print(fmt, ...) do { if(DO_DEBUG_PRINT) printf(fmt, __VA_ARGS__); } while (0)
    #else
        #if ((WARNINGS==1) and (DO_WARNINGS==1))
            #warning No printing
        #endif
        #define debug_print(fmt, ...)
    #endif

    #ifndef DEBUG_PRINTF_BUFSIZE
        #define SPRINTF_LEN 300
    #else
        #define SPRINTF_LEN DEBUG_PRINTF_BUFSIZE // From "debug_conf.h", to here and /lib_logging
    #endif

    /*
     _____      _       _
     |  __ \    (_)     | |
     | |__) | __ _ _ __ | |_   _ __ ___   __ _  ___ _ __ ___  ___
     |  ___/ '__| | '_ \| __| | '_ ` _ \ / _` |/ __| '__/ _ \/ __|
     | |   | |  | | | | | |_  | | | | | | (_| | (__| | | (_) \__ \
     |_|   |_|  |_|_| |_|\__| |_| |_| |_|\__,_|\___|_|  \___/|___/
     TO AVOID THEM CLUTTERING THE CODE
    */

    // None yet
#else
    #error NESTED .h FILE
#endif /* PRINT_MACROS_XC_H_ */
