/*
 * _AudioMux_controller.xc
 *
 *  Created on: 15. aug. 2018
 *      Author: teig
 */

#define INCLUDES
#ifdef INCLUDES
    #include <xs1.h>
    #include <platform.h> // slice
    #include <timer.h>    // delay_milliseconds(200), XS1_TIMER_HZ etc
    #include <stdint.h>   // uint8_t
    #include <stdio.h>    // printf
    #include <string.h>   // memcpy
    #include <xccompat.h> // REFERENCE_PARAM(my_app_ports_t, my_app_ports) -> my_app_ports_t &my_app_ports
    #include <iso646.h>   // not etc.
    #include <i2c.h>
    #include <i2s.h>
    #include <mic_array.h>

    #include "_version.h" // First this..
    #include "_globals.h" // ..then this

    #include "param.h"
    #include "i2c_client_task.h"
    #include "display_ssd1306.h"
    #include "core_graphics_adafruit_gfx.h"
    #include "_texts_and_constants.h"
    #include "button_press.h"
    #include "pwm_softblinker.h"

#endif

#include "AN00220_app_phase_aligned_example.h"
#include "AN00219_headset_output.h"

#include "_Beep_BRRR_01.h"

#if (WARNINGS == 1)
    #if (MIC_ARRAY_NUM_MICS == 8)
        #warning MIC_ARRAY_NUM_MICS = 8
    #elif (MIC_ARRAY_NUM_MICS == 4)
        #warning MIC_ARRAY_NUM_MICS = 4
    #else
        #error
    #endif
#endif


// Observe that I have no control of the ports during xTIMEcomposer downloading
// I have observed a 700-800 ms low on signal pins before my code starts

// ---
// 1 BIT PORT TARGET_XCORE-XA-MODULE
// ---

#if (IS_MYTARGET == IS_MYTARGET_STARTKIT)
    out buffered port:1 outP1_d4_led       = on tile[0]: XS1_PORT_1P; // J7 GPIO23 X0D39 = text D39

    // Open button 3V3 pull-up with 10k, pushed button takes that line via 1k to GND
    in  buffered port:1 inP_button_left    = on tile[0]: XS1_PORT_1O; // External J7 GPIO21 X0D38 = text D38
    in  buffered port:1 inP_button_center  = on tile[0]: XS1_PORT_1I; // External J7 GPIO20 X0D24 = text D24
    in  buffered port:1 inP_button_right   = on tile[0]: XS1_PORT_1L; // External J7 GPIO19 X0D35 = text D35

    port                p_display_scl      = on tile[0]: XS1_PORT_1K; // External J7 GPIO11 X0D34 = text D34
    port                p_display_sda      = on tile[0]: XS1_PORT_1J; // External J7 GPIO10 X0D25 = text D25
    out port            p_display_notReset = on tile[0]: XS1_PORT_1M; // External J7 GPIO15 X0D36 = text D36
                                                                      // Adafruit monochrome 128x32 I2C OLED graphic display PRODUCT ID: 931, containing
                                                                      // module UG-2832HSWEG02 with chip SSD1306 from Univision Technology Inc. Data sheet often says 128 x 64 bits
                                                                      // as it looks like much of the logic is the same as for 128 z 32 bits.
                                                                      // At least 3 us low to reset
    port                p_audiomux_scl     = on tile[0]: XS1_PORT_1H; // External J7 GPIO02 X0D23 = text D23
    port                p_audiomux_sda     = on tile[0]: XS1_PORT_1F; // External J7 GPI001 X0D13 = text D13

#elif (IS_MYTARGET == IS_MYTARGET_XCORE_200_EXPLORER)

    in buffered port:1 inP_button_left     = on tile[0]: XS1_PORT_1M; // External GPIO-PIN63 With pull-up of 9.1k
    in buffered port:1 inP_button_center   = on tile[0]: XS1_PORT_1N; // External GPIO-PIN61 With pull-up of 9.1k
    in buffered port:1 inP_button_right    = on tile[0]: XS1_PORT_1O; // External GPIO-PIN59 With pull-up of 9.1k

#elif (IS_MYTARGET == IS_MYTARGET_XCORE_XA_MODULE)
    out buffered port:1 outP1_d4_led       = on tile[0]: XS1_PORT_1F; // xCORE XA J1 D13 XCORE-XA-MODULE LED D4 (LOW IS ON)

    // Open button 3V3 pull-up with 10k, pushed button takes that line via 1k to GND
    in  buffered port:1 inP_button_left    = on tile[0]: XS1_PORT_1K; // External xCORE XA J9 P34. XCORE-XA-MODULE EXTERNAL BUTTON1
    in  buffered port:1 inP_button_center  = on tile[0]: XS1_PORT_1O; // External xCORE XA J9 P38. XCORE-XA-MODULE EXTERNAL BUTTON2
    in  buffered port:1 inP_button_right   = on tile[0]: XS1_PORT_1P; // External xCORE XA J9 P39. XCORE-XA-MODULE EXTERNAL BUTTON3

    port                p_display_scl      = on tile[0]: XS1_PORT_1H; // External xCORE XA J9 P6
    port                p_display_sda      = on tile[0]: XS1_PORT_1L; // External xCORE XA J9 P8
    out port            p_display_notReset = on tile[0]: XS1_PORT_1G; // External xCORE XA J9 P5 (but the display uses xCORE reset signal P4 instead)
                                                                      // on adafruit monochrome 128x32 I2C OLED graphic display PRODUCT ID: 931, containing
                                                                      // module UG-2832HSWEG02 with chip SSD1306 from Univision Technology Inc. Data sheet often says 128 x 64 bits
                                                                      // as it looks like much of the logic is the same as for 128 z 32 bits.
                                                                      // At least 3 us low to reset
    port                p_audiomux_scl     = on tile[0]: XS1_PORT_1B; // External xCORE XA J1 P6
    port                p_audiomux_sda     = on tile[0]: XS1_PORT_1D; // External xCORE XA J1 P8
#elif (IS_MYTARGET == IS_MYTARGET_MIC_ARRAY)
    //                                                            MIC-ARRAY-1V3-MOD.xc
    on tile[0]: in port p_pdm_clk              = XS1_PORT_1E;  // PORT_PDM_CLK
    on tile[0]: in buffered port:32 p_pdm_mics = XS1_PORT_8B;  // The 8 bit wide port connected to the PDM microphones
                                                               // 7 used: MIC_0_DATA 8B0 to MIC_6_DATA 8B6 (8B7 not connected)
                                                               // Also: "Count of microphones(channels) must be set to a multiple of 4"
                                                               // PORT_PDM_DATA Inputs four chunks of 8 bits into a 32 bits value
    on tile[0]: in port p_mclk                 = XS1_PORT_1F;  // PORT_MCLK_TILE0
    on tile[0]: clock pdmclk                   = XS1_CLKBLK_1; // In "xs1b_user.h" system

    //                                                       MIC-ARRAY-1V3-MOD.xc
    on tile[1]:     port p_i2c             = XS1_PORT_4E; // PORT_I2C             SCL=BIT0, SDA=BIT1
    on tile[1]: out port p_shared_notReset = XS1_PORT_4F; // PORT_SHARED_RESET    BIT0 reset when low
    on tile[1]:     port p_i2s_bclk        = XS1_PORT_1M; // PORT_I2S_BCLK
    on tile[1]:     port p_i2s_lrclk       = XS1_PORT_1N; // PORT_I2S_LRCLK
    on tile[1]:     port p_i2s_dac_data    = XS1_PORT_1P; // PORT_I2S_DAC0

    on tile[0]: out port p_scope_gray      = XS1_PORT_1J; // Mic array expansion header J5, pin 10

    on tile[0]: in buffered port:4 inP_4_buttons = XS1_PORT_4A; // BUTTONS_NUM_CLIENTS
    //
    on tile[0]:     port p_display_scl      = XS1_PORT_1H; // Mic array expansion header J5, pin  3
    on tile[0]:     port p_display_sda      = XS1_PORT_1G; // Mic array expansion header J5, pin  1
    on tile[0]: out port p_display_notReset = XS1_PORT_1A; // Mic array expansion header J5, pin  5 Adafruit 326 v2.1 does not NOT have on-board reset logic
    on tile[0]: out port p_scope_orange     = XS1_PORT_1D; // Mic array expansion header J5, pin  7
    on tile[0]: out port p_scope_violet     = XS1_PORT_1I; // Mic array expansion header J5, pin  9


    on tile[0]: out port leds_00_07 = XS1_PORT_8C;  // BIT0 is LED_0

    on tile[0]: out buffered port:1 led_08 = XS1_PORT_1K;
    on tile[0]: out port led_09            = XS1_PORT_1L;
    on tile[0]: out port led_10            = XS1_PORT_1M;
    on tile[0]: out port led_11            = XS1_PORT_1N;
    on tile[0]: out port led_12            = XS1_PORT_1O;

    // If we need to collect them:
    //out port leds_08_12[NUM_LEDS_08_12] = {XS1_PORT_1K, XS1_PORT_1L, XS1_PORT_1M, XS1_PORT_1N, XS1_PORT_1O};

#endif

#define I2C_DISPLAY_MASTER_SPEED_KBPS  100 // 333 is same speed as used in the aquarium in i2c_client_task.xc,
                                           // i2c_internal_config.clockTicks 300 for older XMOS code struct r_i2c in i2c.h and module_i2c_master
#define I2C_INTERNAL_NUM_CLIENTS 1


#if ((MY_CONFIG == CONFIG_USE_INTERFACE) or (MY_CONFIG == CONFIG_USE_CHAN))

    int data [NUM_DATA_X] [NUM_DATA_Y];

    int main() {

        interface pin_if_1       if_pin    [BUTTONS_NUM_CLIENTS];
        interface button_if_1    if_button [BUTTONS_NUM_CLIENTS];
        interface pwm_if         if_pwm;
        interface softblinker_if if_softblinker;
        i2c_general_commands_if  if_i2c_general_commands;
        i2c_internal_commands_if if_i2c_internal_commands;
        i2c_master_if            if_i2c[I2C_HARDWARE_NUM_BUSES][I2C_HARDWARE_NUM_CLIENTS];

        #if (MY_CONFIG == CONFIG_USE_INTERFACE)
            mic_result_if      if_mic_result;
            #define MIC_RESULT if_mic_result
            #if (WARNINGS == 1)
                #warning MY_CONFIG == CONFIG_USE_INTERFACE
            #endif
        #elif (MY_CONFIG == CONFIG_USE_CHAN)
            chan               c_mic_result;
            #define MIC_RESULT c_mic_result
            #if (WARNINGS == 1)
                #warning MY_CONFIG == CONFIG_USE_CHAN
            #endif
        #endif

        par {
            on tile[0]:{
                configure_clock_src_divide  (pdmclk, p_mclk, 4); // XS1_CLKBLK_1
                configure_port_clock_output (p_pdm_clk, pdmclk);
                configure_in_port           (p_pdm_mics, pdmclk);
                start_clock                 (pdmclk);

                streaming chan c_pdm_to_dec   [NUM_4MIC_DECIMATORS];
                streaming chan c_ds_ptr_output[NUM_4MIC_DECIMATORS]; // chan contains pointers to data and control information
                                                                     // Relies oo shared memory, so both's ends taks must on the same tile as this task
                // mic_array_pdm_rx()
                //     samples up to 8 microphones and filters the data to provide up to eight 384 kHz data streams, split in two streams of four channels.
                //     The gain is corrected so that a maximum signal on the PDM microphone corresponds to a maximum signal on the PCM signal.
                // PDM microphones typically have an initialization delay in the order of about 28ms. They also typically have a DC offset.
                // Both of these will be specified in the datasheet.

                par {
                    #if (MIC_ARRAY_NUM_MICS == 8)
                        mic_array_pdm_rx              (p_pdm_mics, c_pdm_to_dec[0], c_pdm_to_dec[1]);                     // in pdm.xc, calls pdm_rx_asm in pdm_rx.xc which never returns
                        mic_array_decimate_to_pcm_4ch (c_pdm_to_dec[0], c_ds_ptr_output[0], MIC_ARRAY_NO_INTERNAL_CHANS); // asm in decimate_to_pcm_4ch.S
                        mic_array_decimate_to_pcm_4ch (c_pdm_to_dec[1], c_ds_ptr_output[1], MIC_ARRAY_NO_INTERNAL_CHANS); // asm in decimate_to_pcm_4ch.S
                    #elif (MIC_ARRAY_NUM_MICS == 4)
                        mic_array_pdm_rx              (p_pdm_mics, c_pdm_to_dec[0], null);                                // in pdm.xc, calls pdm_rx_asm in pdm_rx.xc which never returns
                        mic_array_decimate_to_pcm_4ch (c_pdm_to_dec[0], c_ds_ptr_output[0], MIC_ARRAY_NO_INTERNAL_CHANS); // asm in decimate_to_pcm_4ch.S
                    #else
                        #error
                    #endif

                    example_task (c_ds_ptr_output, data, p_scope_gray, MIC_RESULT); // chan contains ptr to shared data, must be on the same tile
                                                                                    // as mic_array_decimate_to_pcm_4ch
                }
            }

            par {
                on tile[0]: Handle_Beep_BRRRR_task (
                        if_button, leds_00_07, if_softblinker,
                        if_i2c_internal_commands,
                        if_i2c_general_commands,
                        p_display_notReset,
                        MIC_RESULT);

                // Having this here, and not in combined part below, avoids "line 2303" compiler crash:
                on tile[0].core[0]: I2C_Client_Task_simpler (if_i2c_internal_commands, if_i2c_general_commands, if_i2c);
            }
            on tile[0]: { // Having these share a core avoids "line 183" compiler crash and runs:
                [[combine]]
                par {
                    softblinker_task (if_pwm, if_softblinker);
                    pwm_for_LED_task (if_pwm, led_08);
                    i2c_master (if_i2c[I2C_HARDWARE_IOF_DISPLAY_AND_IO], I2C_HARDWARE_NUM_CLIENTS, p_display_scl, p_display_sda, I2C_DISPLAY_MASTER_SPEED_KBPS); // Synchronous==distributable
                }
            }
            on tile[0]: {
                [[combine]]
                par {
                    Buttons_demux_task (inP_4_buttons, if_pin);
                    Button_task        (IOF_BUTTON_LEFT,      if_pin[IOF_BUTTON_LEFT],      if_button[IOF_BUTTON_LEFT]);      // BUTTON_A
                    Button_task        (IOF_BUTTON_CENTER,    if_pin[IOF_BUTTON_CENTER],    if_button[IOF_BUTTON_CENTER]);    // BUTTON_B
                    Button_task        (IOF_BUTTON_RIGHT,     if_pin[IOF_BUTTON_RIGHT],     if_button[IOF_BUTTON_RIGHT]);     // BUTTON_C
                    Button_task        (IOF_BUTTON_FAR_RIGHT, if_pin[IOF_BUTTON_FAR_RIGHT], if_button[IOF_BUTTON_FAR_RIGHT]); // BUTTON_D
                }
            }
        }

        return 0;
    }
#else
    #error no config
#endif
