// Copyright (c) 2015-2016, XMOS Ltd, All rights reserved
// Teig: Taken from AN00220

// Used by lib_mic_array in:
//                 decimate_to_pcm_4ch.S asm
//                     mic_array_frame.h
//                              pdm_rx.S asm
// Somehow it's also seen by
//    AN00220_app_phase_aligned_example.xc

#ifndef MIC_ARRAY_CONF_H_
    #define MIC_ARRAY_CONF_H_

    #ifndef APPLICATION_NOTE
        #error -DAPPLICATION_NOTE not defined in makefile
    #elif (WARNINGS==1)
        #if (APPLICATION_NOTE == 219)
            #warning AN00219 config inspired // AN00219_headset_output.xc
        #elif (APPLICATION_NOTE == 220)
            #warning AN00220 config inspired // AN00220_app_phase_aligned_example.xc
        #else
            #error Wrong value
        #endif
    #endif

    #if (APPLICATION_NOTE == 219)               // AN00219 config inspired
        #define MIC_ARRAY_MAX_FRAME_SIZE_LOG2 0
        #define DECIMATION_FACTOR             2 // 48 kHz etc.
        #define MIC_ARRAY_NUM_MICS            8 // Must have 8 mics since directive
    #elif (APPLICATION_NOTE == 220)             // AN00220 config inspired
        #define MIC_ARRAY_MAX_FRAME_SIZE_LOG2 3
        #define DECIMATION_FACTOR             6 // 16 kHz etc.
        #define MIC_ARRAY_NUM_MICS            4 // I made it 4 to see how using less worked out
    #endif

    // MIC_ARRAY_MAX_FRAME_SIZE_LOG2
    // 0 : 2 exp(0) = 1 sample  per microphone in audio.data in example
    // 1 : 2 exp(1) = 2 samples per microphone in audio.data in example
    // 2 : 2 exp(2) = 4 samples per microphone in audio.data in example
    // 3 : 2 exp(3) = 8 samples per microphone in audio.data in example
    //
    #define MIC_ARRAY_NUM_SAMPLES (1 << MIC_ARRAY_MAX_FRAME_SIZE_LOG2)

    // The four channel decimators (mic_array_pdm_rx()) output frames of either
    // time domain audio or FFT ready audio prepared for an FFT. The define
    // MIC_ARRAY_MAX_FRAME_SIZE_LOG2 (found in mic_array_conf.h) should be used
    // to allocate the arrays to store the frames. This means that all frames
    // structures will allocate enough memory to allow for a frame size of
    // two to the power of MIC_ARRAY_MAX_FRAME_SIZE_LOG2 regardless of the size
    // used in the decimator_config_common.
    // It is recommended that the frame_size_log2 field of decimator_config_common
    // is always set to MIC_ARRAY_MAX_FRAME_SIZE_LOG2. Equally, the define MIC_ARRAY_NUM_MICS
    // is used for allocating the memory for the frame structure. This must be set to a multiple of 4.

    // The mic_array board has 7 Infineon IM69D130 MEMS microphones
    // "Count of microphones(channels) must be set to a multiple of 4", ie 4 or 8

    // If the decimation factor is changed the the coefs_array of mic_array_decimator_config_t
    // must also be changed. Done below

    // From the PDF doc XM010267
    // 48 kHz, 24 kHz, 16 kHz, 12 kHz and 8 kHz output sample rate by default (3.072MHz PDM clock)
    // Configurable frame size from 1 sample to 8192 samples plus 50% overlapping frames option,
    // chapter 11 "Four Channel Decimator"
    //
    // 96 kHz divided by 2, 4, 6, 8 or 12. See "fir_coefs.h" and "fir_coefs.xc" in lib_mic_array
    #if (DECIMATION_FACTOR == 2)
        #define G_COEFS_ARRAY          g_third_stage_div_2_fir  // [126]
        #define SAMPLING_FREQUENCY_HZ  48000                    // 48 kHz
        #define FIR_COMPENSATOR_DIV_NN FIR_COMPENSATOR_DIV_2
    #elif (DECIMATION_FACTOR == 4)
        #define G_COEFS_ARRAY          g_third_stage_div_4_fir  // [252]
        #define SAMPLING_FREQUENCY_HZ  24000                    // 24 kHz
        #define FIR_COMPENSATOR_DIV_NN FIR_COMPENSATOR_DIV_4
    #elif (DECIMATION_FACTOR == 6)
        #define G_COEFS_ARRAY          g_third_stage_div_6_fir  // [378]
        #define SAMPLING_FREQUENCY_HZ  16000                    // 16 kHz
        #define FIR_COMPENSATOR_DIV_NN FIR_COMPENSATOR_DIV_6
    #elif (DECIMATION_FACTOR == 8)
        #define G_COEFS_ARRAY          g_third_stage_div_8_fir  // [504]
        #define SAMPLING_FREQUENCY_HZ  12000                    // 12 kHz
        #define FIR_COMPENSATOR_DIV_NN FIR_COMPENSATOR_DIV_8
    #elif (DECIMATION_FACTOR == 12)
        #define G_COEFS_ARRAY          g_third_stage_div_12_fir // [756]
        #define SAMPLING_FREQUENCY_HZ  8000                     // 8 kHz
        #define FIR_COMPENSATOR_DIV_NN FIR_COMPENSATOR_DIV_12
    #else
        #error See
    #endif

#else
    #error NESTED .h FILE
#endif /* MIC_ARRAY_CONF_H_ */
